#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "list.h"

link_t* link_init(void* data)
{
	link_t*	link=malloc(sizeof(link_t));

  assert(link != NULL);

  link->next = NULL;
  link->data = data;

  return(link);
}

list_t* list_init(void)
{
  	list_t	*hdr=NULL;

  // malloc new list header and init
  hdr = malloc(sizeof(list_t));

  assert(hdr != NULL);

  // NULL pointers indicate empty list
  hdr->first = hdr->last = hdr->current = NULL;

  return(hdr);
}

int list_empty(list_t *list)
{
  return list->first == NULL ? 1 : 0;
}


void list_add(list_t *list,void *data)
{
	link_t	*link = link_init(data);

  // add an element to end of a list
  if(list_empty(list)) {
    list->first = link;
    list->last = link;
  } else {
    list->last->next = link;
    list->last = link;
  }
  // set iterator to last element added
  list->current = link;
  return;
}


void list_del_front_link(list_t *list)
{
	link_t	*link;

  // get pointer to first element, make sure it's not NULL
  assert((link = list->first) != NULL);
  list->first = list->first->next;	// make first point to second
  list_reset(list);			// get current to point to first
  free(link->data);			// free user data (nasty!)
  free(link);				// free link
}

void list_del(list_t *list)
{
  // get current to point to first
  list_reset(list);

  // delete all list structures and items
  while(list_not_end(list)) list_del_front_link(list);

  free(list);
}

void list_reset(list_t *list)
{
  // current points to first
  list->current = list->first;
}

int list_not_end(list_t *list)
{
  if(list->current != NULL)	return(1);
  else				return(0);
}

void list_next_link(list_t *list)
{
  // get current to point to next link
  assert(list->current != NULL);
  list->current = list->current->next;
}

void *list_get_data(list_t *list)
{
  assert(list->current != NULL);
  return(list->current->data);
}

