#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "vector.h"
#include "pixel.h"
#include "list.h"
#include "camera.h"
#include "material.h"
#include "object.h"
#include "model.h"

// create a new model description
model_t* model_init(FILE *in)
{
	model_t		*model;
	material_t	*mat;
	object_t	*obj;
	char		token[16];

  // create new object and zero it
  assert((model = (model_t *)malloc(sizeof(model_t))) != NULL);
  memset(model,0,sizeof(model_t));

  // create material and object lists
  model->mats = list_init();
  model->objs = list_init();

  // input should consist of material definitions followed by
  // object definitions
  while(fscanf(in,"%s",token) == 1) {

    if(!strcmp(token,"camera")) {
      camera_init(in,&(model->cam),0);

      // this test is designed to ensure that we just read
      // the camera
      assert(model->cam->cookie == CAM_COOKIE);
      fprintf(stderr,"loaded %s \n",camera_getname(model->cam));
    }

    if(!strcmp(token,"material")) {
      // create material_t structure and read attributes
      material_init(in,model->mats,0);

      // this test is designed to ensure that list_add
      // pointed current to the material just loaded
      mat = (material_t *)list_get_data(model->mats);
      assert(mat->cookie == MAT_COOKIE);
      fprintf(stderr,"loaded %s \n",material_getname(mat));
    }

    if(!strcmp(token,"plane")) {
      // create plane object
      plane_init(in,model->objs,model->mats,0);

      // this test is designed to ensure that list_add
      // pointed current to the material just loaded
      obj = (object_t *)list_get_data(model->objs);
      assert(obj->cookie == OBJ_COOKIE);
      fprintf(stderr,"loaded %s \n",object_getname(obj));

    }
  }

  return(model);
}

void model_print(model_t *model,FILE* out)
{
  // verify that we have the camera
  if(model->cam) camera_print(model->cam,out);

  // verify that we have all materials
  material_list_print(model->mats,out);

  // verify that we have all objects
  object_list_print(model->objs,out);
}

void model_del(model_t *model)
{
  camera_del(&model->cam);
  list_del(model->objs);
  list_del(model->mats);

  free(model);
}
