#ifndef NAME_LEN
#define NAME_LEN   16        // max length of entity names
#endif

#define OBJ_COOKIE 12345678

typedef struct object_type
{
  int        cookie;
  char       type[NAME_LEN];	// e.g., plane, sphere, etc.
  char       name[NAME_LEN];	// left_wall, center_sphere, etc.

  material_t *mat;		// material

  void       (*printer)(struct object_type*,FILE*);
  double     (*hits)(struct object_type*,vec_t,vec_t);

  void       (*ambient) (material_t *,drgb_t);
  void       (*diffuse) (material_t *,drgb_t);
  void       (*specular)(material_t *,drgb_t);

  vec_t      last_hit;		// last hit point
  vec_t      last_normal;	// normal at last hit point
} object_t;

// object_init signature changes to include object_t* coming in,
// but this actually is more consistent usage w.r.t. remaining functions
void   object_init(object_t*,FILE*,list_t*,list_t*);
double object_no_hit(object_t*,vec_t,vec_t);
void   object_list_print(list_t*,FILE*);
void   object_print(object_t*,FILE*);
char*  object_getname(object_t*);

